@@ -7,7 +7,9 @@ from collective.dms.batchimport.utils import log
 from collective.dms.mailcontent.dmsmail import internalReferenceIncomingMailDefaultValue
 from collective.zamqp.consumer import Consumer
 from imio.dms.mail import IM_EDITOR_SERVICE_FUNCTIONS
+from imio.dms.mail.utils import create_period_folder
 from imio.dms.mail.utils import get_dms_config
+from imio.dms.mail.utils import sub_create
 from imio.helpers.content import transitions
 from imio.helpers.security import get_user_from_criteria
 from imio.zamqp.core import base
@@ -17,10 +19,13 @@ from io import BytesIO
 from plone import api
 from plone.dexterity.utils import createContentInContainer
 from plone.namedfile.file import NamedBlobFile
+from Products.CMFCore.WorkflowCore import WorkflowException
 from Products.CMFPlone.utils import base_hasattr
 from Products.CMFPlone.utils import safe_unicode
 from z3c.relationfield.relation import RelationValue
 from zope.component import getUtility
+from zope.globalrequest import getRequest
+from zope.i18n import translate
 from zope.intid.interfaces import IIntIds
 from zope.schema.interfaces import IVocabularyFactory
 
@@ -82,7 +87,7 @@ class IncomingMail(DMSMainFile, CommonMethods):
         self.creating_group_split()
         (document, main_file) = createDocument(
             self.context,
-            self.folder,
+            create_period_folder(self.folder, datetime.datetime.now()),
             self.document_type,
             '',
             obj_file,
@@ -147,7 +152,7 @@ class OutgoingMail(DMSMainFile, CommonMethods):
         self.creating_group_split()
         (document, main_file) = createDocument(
             self.context,
-            self.folder,
+            create_period_folder(self.folder, datetime.datetime.now()),
             self.document_type,
             '',
             obj_file,
@@ -355,18 +360,23 @@ class IncomingEmail(DMSMainFile, CommonMethods):
         for key in ('scanner', 'scan_user', 'pages_number'):
             del self.scan_fields[key]
 
-        self.metadata['title'] = maildata['Subject']
+        self.metadata['title'] = maildata['Subject'] or u'(VIDE)'
+        # or translate(u'(EMPTY)', domain='imio.zamqp.dms', context=getRequest())
         if 'internal_reference_no' not in self.metadata:
             self.metadata['internal_reference_no'] = internalReferenceIncomingMailDefaultValue(self.context)
         if self.scan_fields['scan_date']:
             self.metadata['reception_date'] = self.scan_fields['scan_date']
-        mail_types = api.portal.get_registry_record('imio.dms.mail.browser.settings.IImioDmsMailConfig.mail_types')
-        if u'email' in [dic['value'] for dic in mail_types if dic['active']]:
+        mail_types_rec = api.portal.get_registry_record('imio.dms.mail.browser.settings.IImioDmsMailConfig.mail_types')
+        mail_types = [dic['value'] for dic in mail_types_rec if dic['active']]
+        if u'email' in mail_types:
             self.metadata['mail_type'] = u'email'
+        elif u'courrier' in mail_types:
+            self.metadata['mail_type'] = u'courrier'
 
         intids = getUtility(IIntIds)
         with api.env.adopt_user(user=api.user.get_current()):
-            document = createContentInContainer(self.folder, 'dmsincoming_email', **self.metadata)
+            document = sub_create(self.folder, 'dmsincoming_email', datetime.datetime.now(), self.metadata.pop('id'),
+                                  **self.metadata)
             log.info('document has been created (id: %s)' % document.id)
             catalog = api.portal.get_tool('portal_catalog')
 
@@ -417,10 +427,13 @@ class IncomingEmail(DMSMainFile, CommonMethods):
             # u'agent', title=_(u'A user forwarded email will go to agent level')),
             if fw_tr != 'created':
                 to_state = 'created'
-                if fw_tr == 'manager':
-                    to_state = 'proposed_to_manager'
-                elif document.treating_groups:
-                    if fw_tr == 'agent' or '_n_plus_1' not in get_dms_config(['review_levels', 'dmsincomingmail']):
+                if document.treating_groups:
+                    trs = ['propose_to_n_plus_1', 'propose_to_n_plus_2', 'propose_to_n_plus_3', 'propose_to_n_plus_4',
+                           'propose_to_n_plus_5', 'propose_to_manager', 'propose_to_pre_manager']
+                    if fw_tr == 'manager':
+                        to_state = 'proposed_to_manager'
+                        trs = ['propose_to_manager', 'propose_to_pre_manager']
+                    elif fw_tr == 'agent' or '_n_plus_1' not in get_dms_config(['review_levels', 'dmsincomingmail']):
                         to_state = 'proposed_to_agent'
                     else:
                         to_state = 'proposed_to_agent'
@@ -435,16 +448,23 @@ class IncomingEmail(DMSMainFile, CommonMethods):
                                 while tr != 'propose_to_agent':
                                     to_state = st_from_tr[tr]
                                     tr = tr_levels[to_state][document.treating_groups][0]
+                    if to_state == 'proposed_to_agent':
+                        trs.insert(0, 'propose_to_agent')
                 # we store a flag to indicate that this content is agent forwarded and has been transitioned to
                 if to_state != 'created':
                     setattr(document, '_iem_agent', to_state)
                 i = 0
                 state = api.content.get_state(document)
+                pw = api.portal.get_tool("portal_workflow")
                 while state != to_state and i < 10:
-                    transitions(document, ['propose_to_agent', 'propose_to_n_plus_1', 'propose_to_n_plus_2',
-                                           'propose_to_n_plus_3', 'propose_to_n_plus_4', 'propose_to_n_plus_5',
-                                           'propose_to_manager', 'propose_to_pre_manager'])
-                    state = api.content.get_state(document)
+                    for tr in trs:
+                        try:
+                            pw.doActionFor(document, tr)
+                        except WorkflowException:
+                            continue
+                        state = api.content.get_state(document)
+                        if state == to_state:
+                            break
                     i += 1
 
             file_object = NamedBlobFile(mf_tup[1], filename=mf_tup[0])
